// NOContentStreamNavigator.h
//
// NOContentStreamNavigator: navigate the (raw) contentstream
//			
// (c) NiXPS NV, April 2006

#ifndef _NOCONTENTSTREAMNAVIGATOR_H_
#define _NOCONTENTSTREAMNAVIGATOR_H_

/*! \file NOContentStreamNavigator.h
    \brief Header file for NOContentStreamNavigator class
*/

#include "NTypes.h"

extern "C" {
	DLLIMPORTEXPORT NOContentStreamNavigatorHandler NOContentStreamNavigator_new(NOHandler *pHandler);
	DLLIMPORTEXPORT void NOContentStreamNavigator_getNodeInfo(NOContentStreamNavigatorHandler *pHandler, NCommon::UInt32 *lPos, NCommon::UInt32 *lLen);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_getNumberOfChilds(NOContentStreamNavigatorHandler *pHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_toChild(NOContentStreamNavigatorHandler *pHandler, NCommon::UInt32 pNum);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_toChildWithName(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pStringHandler, NCommon::UInt32 pCaseSensitive, NCommon::UInt32 pIgnorePrefix);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_toSibling(NOContentStreamNavigatorHandler *pHandler);
	DLLIMPORTEXPORT void NOContentStreamNavigator_getName(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pStringHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_hasAttributeValue(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pKey);
	DLLIMPORTEXPORT void NOContentStreamNavigator_getAttributeValue(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pKey, NOStringHandler *pOutValue);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_setAttributeValue(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pKey, NOStringHandler *pValue);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_addAttribute(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pKey, NOStringHandler *pValue);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_toAttributeValue(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pKey);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_hasCharacterData(NOContentStreamNavigatorHandler *pHandler);
	DLLIMPORTEXPORT void NOContentStreamNavigator_getCharacterData(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pStringHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_insertChild(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pName, NCommon::UInt32 pPosition, NCommon::UInt32 pToChild);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_insertAsChild(NOContentStreamNavigatorHandler *pHandler, NOStringHandler *pXML);
	DLLIMPORTEXPORT NCommon::UInt32 NOContentStreamNavigator_insertAsChildWithNavigator(NOContentStreamNavigatorHandler *pHandler, NOContentStreamNavigatorHandler *pNav);
};

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

class NOXMLPartBased;

/*!
 * \class NOContentStreamNavigator NOContentStreamNavigator.h "NOContentStreamNavigator.h"
 * Use an NOContentStreamNavigator to navigate and manipulate an XML content stream
 */
class NOContentStreamNavigator
{
	private:
		mutable NOContentStreamNavigatorHandler mHandler;
			
	public:
		// constructor: object is always linked to a specific NAPackage
		NOContentStreamNavigator(NOContentStreamNavigatorHandler &pHandler);

		// constructor: object is always linked to a specific NAPackage
		NOContentStreamNavigator(NOXMLPartBased &pPart);

		// destructor
		virtual ~NOContentStreamNavigator();		

		/// Get Position and Length of the current node
		/// @param lPos On return, the position of the current node
		/// @param lLen On return, the length of the current node
		void getNodeInfo(NCommon::UInt32 &lPos, NCommon::UInt32 &lLen);

		/// Get the number of child nodes
		NCommon::UInt32 getNumberOfChilds() const;						

		/// Navigate to child node at given index
		/// @param pNum the index of the node to navigate to
		bool toChild(const NCommon::UInt32 pNum);

		/// Navigate to child node with given name, optionally ingnoring case and/or namespace
		/// @param pString The name of the node to go to
		/// @param pCaseSensitive Behave case sensitive.  Defaults to true (case sensitive)
		/// @param pIgnorePrefix Ignore namespace prefixes.  Defaults to true (ignore namespace prefixes)
		bool toChild(const NCommon::UTF8String &pString, const bool pCaseSensitive=true, const bool pIgnorePrefix=true);

		/// Navigate to next sibling
		/// @return True upon success, false if there is no next sibling
		bool toSibling();

		/// Return the name of the current node as a UTF8String
		const NCommon::UTF8String getName() const;
		
		/// Test for the presence of the given attribute
		/// @param pAttributeName The attribute name to test for
		bool hasAttributeValue(const NCommon::UTF8String &pAttributeName) const;

		/// Get the value of an attribute of the current node
		/// @param pAttributeName The name of the attribute to get the value of
		/// @return The value of the attribute as a (const) UTF8String
		const NCommon::UTF8String getAttributeValue(const NCommon::UTF8String &pAttributeName) const;

		/// Set the value of an attribute in the current node
		/// @param pAttributeName The name of the attribute to set the value of
		/// @param pValue The string to set the attribute value to
		bool setAttributeValue(const NCommon::UTF8String &pAttributeName, const NCommon::UTF8String &pValue);

		/// Adds an attribute to the current node
		/// @param pAttributeName The name of the attribute to set the value of
		/// @param pValue The string to set the attribute value to
		bool addAttribute(const NCommon::UTF8String &pKey, const NCommon::UTF8String &pValue);

		/// Navigate to the attribute value
		/// @param pAttributeName The name of the attribute to set the value of
		bool toAttributeValue(const NCommon::UTF8String &pAttributeName);

		/// Check to see if the current element has character data
		bool hasCharacterData() const;

		/// Get the character data for the current node as a UTF8String
		const NCommon::UTF8String getCharacterData() const;

		/// Inserts a child in the current node
		/// @param pName The name of the element as a UTF8String
		/// @param pPosition The position of this inserted element
		/// @param pToChild Navigate to this freshly created child, or not
		/// @return Whether the operation was successful
		bool insertChild(const NCommon::UTF8String &pName, NCommon::UInt32 pPosition, bool pToChild);

		/// Inserts XML as a child of the current node
		/// @param pXML The XML to insert as a UTF8String
		/// @return Whether the operation was successful
		bool insertAsChild(const NCommon::UTF8String &pXML);

		/// Inserts XML at the cursor of another content stream navigator as a child of the node at the cursor of this content stream navigator
		/// @param pNav the content stream navigator pointing to the XML node to insert as a child in the current content stream
		/// @return Whether the operation was successful
		bool insertAsChild(NOContentStreamNavigator &pNav);
};

}
#endif // NIXPS_CPP_INTERFACE

#endif
