// NOPackage.h
//
// Object that represents a complete xps package
//			
//
// (c) NiXPS NV, October 2006

/*! \file NOPackage.h
    \brief Header file for NOPackage class
*/

#ifndef _NOXPSPACKAGE_H_
#define _NOXPSPACKAGE_H_

#include "NTypes.h"
#include "NOProgressReporter.h"
#include "NOPartWithResources.h"

// C api
extern "C"
{	
	DLLIMPORTEXPORT NOPackageHandler NOPackage_create(NCommon::UInt32 pCreateEmptyDoc, NCommon::UTF8Char *pName);
	DLLIMPORTEXPORT void NOPackage_destroyPackage(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOPackageHandler NOPackage_readPackageFromFile(NCommon::UTF8Char *pFilename);
	DLLIMPORTEXPORT void NOPackage_writePackageToFile(NOPackageHandler* pPackage, NCommon::UTF8Char *pFilename);
	DLLIMPORTEXPORT void NOPackage_save(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT void NOPackage_getFilename(NOPackageHandler* pPackage, NOStringHandler* pString);
	DLLIMPORTEXPORT void NOPackage_setFilename(NOPackageHandler* pPackage, NCommon::UTF8Char* pString);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_hasFilename(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getDirty(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_hasCoreProperties(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOCorePropertiesHandler NOPackage_getCoreProperties(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOCorePropertiesHandler NOPackage_createCoreProperties(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getNumberOfDocuments(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NODocumentHandler NOPackage_getDocument(NOPackageHandler* pPackage, NCommon::UInt32 pNum);
	DLLIMPORTEXPORT void NOPackage_addDocument(NOPackageHandler* pPackage, NODocumentHandler pDoc, NOProgressReporterHandler pReporter);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getDocumentNumberOf(NOPackageHandler* pPackage, NODocumentHandler pDoc);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_hasThumbnail(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOThumbnailHandler NOPackage_getThumbnail(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getNumberOfPages(NOPackageHandler* pPackage);
}

#ifdef NIXPS_CPP_INTERFACE
// C++ api
namespace NiXPSAccess
{
class NAPackage;
class NAPackageInfo;
}

namespace NiXPSObjects
{

class NODocument;
class NOCoreProperties;
class NODocument;

using namespace NiXPSAccess;

/*!
 * \class NOPackage NOPackage.h "NOPackage.h"
 * Use an NOPackage to read an XPS-file from disk and access or manipulate the content of the package.
 */
class NOPackage: public NOPartWithResources
{
	private:
		// constructor
		// NOPackage(NAPackage &pPackage, const UInt16 &pID);
		NOPackage(NOPackageHandler pHandler);

		// destructor
		virtual ~NOPackage();

	public:		
		/// @name Constructing and destructing an NOPackage
		//@{

		/// Reads a package from a file.
		/// @param pFilename Full path to the XPS-file to read.
		static NOPackage * readPackageFromFile(const UTF8String &pFilename);

		/// Creates an empty package.
		/// @param pCreateEmptyDoc Create an empty document in the package
		static NOPackage * createPackage(bool pCreateEmptyDoc, const UTF8Char *pName=0);
		
		/// Destroy a package.  You need to call this to dispose of a package and all the resources it keeps.
		/// @param pXPSPackage Package to destroy
		static void destroyPackage(NOPackage *pXPSPackage);

		//@}
				
		/// Writes the package to a file
		/// @param pFilename Full path to the XPS-file to write.
		void writePackageToFile(const UTF8String &pFilename);

		/// Writes the package to a file
		void writePackageToFile();

		// returns true if we have a filename
		bool hasOutputFilename() const;

		// returns the filename
		const UTF8String getOutputFilename() const;
		
		// sets the output filename
		void setOutputFilename(const UTF8Char *pFilename);

		void exportToPDF(const UTF8String &pPDFFileName);

		/// Returns whether the package is dirty
		bool getDirty();

		/// Returns whether we have a core properties object
		bool hasCoreProperties();

		/// Returns the NOCoreProperties object, an object represinting the Core Properties of the XPS Package.
		NOCoreProperties getCoreProperties();

		/// @name Manipulating documents in a package
		//@{ 
		// == Document related members
		
		/// Returns the number of documents in this package
		NCommon::UInt32 getNumberOfDocuments();

		/// Returns the document number of a given NODocument in this xps package
		/// @param pDoc The NODocument that the library needs to find the Document Number of.
		NCommon::UInt32 getDocumentNumberOf(const NODocument &pDoc);

		/// Returns the NODocument at postion pNum.
		/// @param pNum The document position
		NODocument getDocument(const NCommon::UInt32 &pNum);

		/// Add a document to the package
		void addDocument(const NODocument &pDoc, NOProgressReporter &pReporter);
		//@}

		// == Thumbnail related members

		/// Returns whether we have a thumbnail
		virtual bool hasThumbnail();

		/// Returns the thumbnail represented by a NOThumbnail object
		NOThumbnail getThumbnail();

		/// Returns the total number of pages in all documents
		NCommon::UInt32 getNumberOfPages();
};

}
#endif // NIXPS_CPP_INTERFACE
#endif
