// NOPage.h
//
// NOPage: NOPage class
//			
//
// (c) NiXPS NV, November 2006

/*! \file NOPage.h
    \brief Header file for NOPage class
*/

#ifndef _NOPAGE_H_
#define _NOPAGE_H_

#include "NTypes.h"
#include "NOPartWithResources.h"
#include "NOThumbnail.h"
#include "NOPackage.h"
#include "NODocument.h"
#include "NOFont.h"
#include "NOImage.h"
#include "NOFontVector.h"
#include "NOImageVector.h"
#include "NOXPrefix.h"

// C api
extern "C"
{
	DLLIMPORTEXPORT NOPageHandler NOPage_new(NOHandler* pHandler, NCommon::UInt16 pID);
	DLLIMPORTEXPORT NODocumentHandler NOPage_getDocument(NOPageHandler* pPage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_getFonts(NOPageHandler* pPage, NOVectorHandler pFonts);
	DLLIMPORTEXPORT NOFontHandler NOPage_getFont(NOPageHandler* pPage, NCommon::UTF8Char *pURI);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_getImages(NOPageHandler* pPage, NOVectorHandler pImages);
	DLLIMPORTEXPORT NOImageHandler NOPage_getImage(NOPageHandler* pPage, NCommon::UTF8Char *pURI);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_getPageNumber(NOPageHandler* pPage);
	DLLIMPORTEXPORT NOFontHandler NOPage_addFont(NOPageHandler* pPage, NOFontHandler pFont);
	DLLIMPORTEXPORT NOFontHandler NOPage_addFontFile(NOPageHandler* pPage, NCommon::UTF8Char *pFontFile);
	DLLIMPORTEXPORT NOImageHandler NOPage_addImage(NOPageHandler* pPage, NOImageHandler pImage);
	DLLIMPORTEXPORT void NOPage_copyResourcesFrom(NOPageHandler* pPage, NOPageHandler* pFromPage);
	DLLIMPORTEXPORT void NOPage_normalizeContentStream(NOPageHandler* pPage);
	DLLIMPORTEXPORT void NOPage_getPageSize(NOPageHandler* pPage, double &pWidth, double &pHeight);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_renderToImage(NOPageHandler *pPage, unsigned char *pImageBuffer,
							NCommon::UInt32 pWidth, NCommon::UInt32 pHeight, NCommon::UInt32  pUseAntiAliasing);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_renderToTiff(NOPageHandler *pPage, NCommon::UTF8Char *pFile, 
							NCommon::UInt32 pWidth, NCommon::UInt32 pHeight, NCommon::UInt32 pResolutionInPixelsPerInch, NCommon::UInt32 pAntiAliasing);
	DLLIMPORTEXPORT NOXHandler NOPage_getFixedPage(NOHandler* pHandler);
	DLLIMPORTEXPORT NOXHandler NOPage_createFixedPage(NOHandler* pHandler);
}

#ifdef NIXPS_CPP_INTERFACE
// C++ api
namespace NiXPSObjects
{

using namespace NCommon;

class NiXPSAccess::NAPackage;

/*!
 * \class NOPage NOPage.h "NOPage.h"
 * Class representing a page in an XPS package.
 */
class NOPage: public NOPartWithResources
{
	private:
		
	public:
		NOPage();
		
		NOPage::NOPage(const NOPageHandler pHandler);

		virtual ~NOPage();

		/// Returns the count of the number of fonts on this page and a vector with all the fonts.
		/// @param pFonts The vector to put the fonts in
		UInt32 getFonts(NOFontVector &pFonts);

		/// Returns the font corresponding to the URI
		/// @param pURI The URI of the font to get as a zero terminated UTF8Char pointer
		NOFont getFont(const UTF8Char *pURI);

		/// Add a font to this page and returns a handler to the new font
		/// @param pFont The font to add
		NOFont addFont(const NOFont &pFont);

		/// Add a font to this page and returns a handler to the new font
		/// @param pFontFile The font file to add as font
		NOFont addFont(const UTF8Char *pFontFile);

		/// Returns the count of the number of images on this page and a vector with all the images
		/// @param pImages The vector to put the images in
		UInt32 getImages(NOImageVector &pImages);
		
		/// Returns the image corresponding to the URI
		/// @param pURI The URI of the image to get as a UTF8String
		NOImage getImage(const UTF8String &pURI);
		
		/// Add an image to this page and returns a handler to the new image
		/// @param pImage The image to add
		NOImage addImage(const NOImage &pImage);

		/// Get the page size
		/// @param pWidth A reference to a float to fill in with the width.
		/// @param pHeight A reference to a float to fill in with the height.
		void getPageSize(double &pWidth, double &pHeight);

		/// Get the page number of this page
		UInt32 getPageNumber() const;

		/// Returns the document this page belongs to
		NODocument getDocument() const;

		/// Copy over the resources from another page to this page
		/// @param pPage the Page to copy resources from
		void copyResourcesFrom(const NOPage &pPage);

		/// Normalize the content stream
		void normalizeContentStream();

		/// get the entry point to the object model for this page
		NOXFixedPage getFixedPage();

		/// create a Fixed page element, this is the entry point to the object model for this page
		NOXFixedPage createFixedPage();

		/// Renders a page to an image buffer of specified width and height
		/// The renderer will put the image centralized in the image buffer if the aspect ratio of the buffer does
		/// not correspond to the aspect ratio of the page.
		/// @pBuffer Image buffer large enough to hold 3 bytes per pixel for an image of the specified width and height
		/// @pWidth The width of the image to render to
		/// @pHeight The height of the image to render to
		/// @pAntiAliasing True or false indicating whether we should do (computationally intensive) antialiasing
		UInt32 renderToImage(unsigned char *pBuffer, UInt32 pWidth, UInt32 pHeight, bool pAntiAliasing);

		/// Renders a page to an tiff file of specified width and height
		/// The renderer will put the image centralized if the aspect ratio of the buffer does
		/// not correspond to the aspect ratio of the page.
		/// @pFile Output Tiff file
		/// @pWidth The width of the image to render to
		/// @pHeight The height of the image to render to
		/// @pResolutionInPixelsPerInch Resolution to be used in the file
		/// @pAntiAliasing True or false indicating whether we should do (computationally intensive) antialiasing
		UInt32 renderToTiff(const UTF8Char *pFile, UInt32 pWidth, UInt32 pHeight, UInt32 pResolutionInPixelsPerInch, UInt32 pAntiAliasing);
};

}
#endif // NIXPS_CPP_INTERFACE

#endif
